@@ -13,7 +13,7 @@ from rdkit.Chem import Draw, AllChem, rdMMPA, rdDepictor
 
 from rdworks.mol import Mol, rd_descriptor, rd_descriptor_f
 from rdworks.mollibr import MolLibr
-from rdworks.display import render_svg
+from rdworks.view import render_svg
 
 
 def get_attachment_orientation(mol: Chem.Mol, dummy_atom_idx: int | None = None):
