@@ -17,9 +17,9 @@ from rdkit.SimDivFilters.rdSimDivPickers import MaxMinPicker
 from PIL import Image
 
 from rdworks import Conf, Mol
-from rdworks.display import render_matrix_grid
+from rdworks.view import render_matrix_grid
 from rdworks.xml import list_predefined_xml
-
+from rdworks.fingerprint import distance_function, calculate_fingerprints
 
 class MolLibr:
     def __init__(self, 
@@ -80,15 +80,6 @@ class MolLibr:
                 self.rename(prefix='entry')
     
 
-    def copy(self) -> Self:
-        """Returns a copy of self.
-
-        Returns:
-            Self: rdworks.MolLibr object.
-        """
-        return copy.deepcopy(self)
-    
-    
     def __str__(self) -> str:
         """Returns string representation.
 
@@ -347,6 +338,50 @@ class MolLibr:
         """
         return mol.qed(properties)
 
+
+    @staticmethod
+    def _mcs_coord_map(subject:Mol, r:Chem.Mol) -> dict:
+        s = subject.rdmol
+        lcs = rdFMCS.FindMCS([r, s])
+        # reference matching indices
+        r_indices = r.GetSubstructMatch(lcs.queryMol)
+        # subject matching indices
+        s_indices = s.GetSubstructMatch(lcs.queryMol)
+        # reference matching coordinates (2D)
+        r_xy = []
+        for i in r_indices:
+            pt = r.GetConformer().GetAtomPosition(i)
+            r_xy.append(Geometry.Point2D(pt.x, pt.y))
+        coord_map = { i : xy for i, xy in zip(s_indices, r_xy) }
+        
+        return coord_map
+    
+
+    @staticmethod
+    def _mask_nnp_ready(mol: Mol, model: str) -> bool:
+        """A mask function to return True if molecule is NNP ready. 
+
+        Args:
+            mol (Mol): rdworks.Mol object.
+            model (str): name of NNP model.
+
+        Returns:
+            bool: True if molecule is NNP ready.
+        """
+        return mol.is_nnp_ready(model)
+    
+
+    ##################################################
+    ### Pipeline Functions (returns Self)
+    ##################################################
+
+    def copy(self) -> Self:
+        """Returns a copy of self.
+
+        Returns:
+            Self: rdworks.MolLibr object.
+        """
+        return copy.deepcopy(self)
     
     def compute(self, **kwargs) -> Self:
         """Change settings for parallel computing.
@@ -552,22 +587,44 @@ class MolLibr:
         return obj
     
 
-    @staticmethod
-    def _mcs_coord_map(subject:Mol, r:Chem.Mol) -> dict:
-        s = subject.rdmol
-        lcs = rdFMCS.FindMCS([r, s])
-        # reference matching indices
-        r_indices = r.GetSubstructMatch(lcs.queryMol)
-        # subject matching indices
-        s_indices = s.GetSubstructMatch(lcs.queryMol)
-        # reference matching coordinates (2D)
-        r_xy = []
-        for i in r_indices:
-            pt = r.GetConformer().GetAtomPosition(i)
-            r_xy.append(Geometry.Point2D(pt.x, pt.y))
-        coord_map = { i : xy for i, xy in zip(s_indices, r_xy) }
+    def pick_diverse(self, n:int, seed:int=0, fp_type:str='morgan', radius:int=2, n_bits:int=2048) -> Self:
+        """
+        Select diverse molecules using MaxMin algorithm.
         
-        return coord_map
+        Args:
+            n (int): Number of molecules to select.
+            seed (int, optional): Random seed for reproducibility. Defaults to 0.
+            fp_type (str, optional): Type of fingerprint to use: morgan, rdkit, maccs. Defaults to 'morgan'
+            radius (int, optional): Radius for Morgan fingerprints. Defaults to 2
+            n_bits (int, optional): Number of bits for fingerprints. Defaults to 2048
+        
+        Returns:
+            List of selected indices
+        """
+        # Calculate fingerprints
+        fps = calculate_fingerprints(self.libr, fp_type, radius, n_bits)
+        
+        # Remove None values (invalid molecules)
+        valid_indices = [i for i, fp in enumerate(fps) if fp is not None]
+        valid_fps = [fp for fp in fps if fp is not None]
+        
+        if len(valid_fps) < n:
+            raise ValueError(f"Requested {n} molecules but only {len(valid_fps)} valid")
+        
+        # Create distance function
+        def dist_func(i, j):
+            return distance_function(i, j, valid_fps)
+        
+        # Run MaxMin picker
+        picker = MaxMinPicker()
+        picks = picker.LazyBitVectorPick(dist_func, len(valid_fps), n, seed=seed)
+        
+        original_indices = [valid_indices[i] for i in picks]
+        mask = [True for i in range(self.count()) if i in original_indices]
+        
+        self.libr = list(itertools.compress(self.libr, mask))
+
+        return self
 
 
     def align_drawing(self, 
@@ -642,27 +699,39 @@ class MolLibr:
         return obj
 
 
-    def pick(self, n: int, **kwargs) -> Self:
-        """Picks n diverse molecules.
+    def nnp_ready(self, model: str, **kwargs) -> Self:
+        """Returns a copy of subset of library that is ready to given neural network potential.
+
+        Examples:
+            >>> libr = rdworks.MolLibr(drug_smiles, drug_names)
+            >>> ani2x_compatible_subset = libr.nnp_ready('ANI-2x', progress=False)
 
         Args:
-            n (int): number of molecules to pick.
+            model (str): name of model.
 
         Returns:
-            Self: a copy of self.
+            Self: subset of library.
         """
-        obj = self.copy()
-        raise NotImplementedError
+        obj = self.copy().compute(**kwargs)
+        lmodel = [model,] * self.count()
+        with ProcessPoolExecutor(max_workers=obj.max_workers) as executor:
+            if obj.progress:
+                mask = list(tqdm(
+                    executor.map(self._mask_nnp_ready, obj.libr, lmodel, chunksize=obj.chunksize),
+                    desc="NNP ready",
+                    total=obj.count()))
+            else:
+                mask = list(
+                    executor.map(self._mask_nnp_ready, obj.libr, lmodel, chunksize=obj.chunksize))
+            obj.libr = list(itertools.compress(obj.libr, mask))
+        
         return obj
-    
-
 
 
     ##################################################
     ### endpoints
     ##################################################
 
-    
     def count(self) -> int:
         """Returns number of molecules.
 
@@ -713,7 +782,6 @@ class MolLibr:
         
         return clusters
     
-
     
     def to_sdf(self, 
                 path: str | Path, 
@@ -817,7 +885,6 @@ class MolLibr:
         return svg_string
 
 
-
     def to_png(self,
                filename: str | Path | None = None,
                mols_per_row: int = 5,
@@ -872,11 +939,11 @@ class MolLibr:
         return HTML
 
 
-    def to_dataframe(self, 
-                     name: str = 'name', 
-                     smiles: str = 'smiles', 
-                     confs: bool = False,
-                     ) -> pd.DataFrame:
+    def to_df(self, 
+            name: str = 'name', 
+            smiles: str = 'smiles', 
+            confs: bool = False,
+            ) -> pd.DataFrame:
         """Returns a Pandas DataFrame.
 
         Args:
@@ -939,57 +1006,14 @@ class MolLibr:
             confs (bool, optional): whether to include conformer properties. Defaults to False.
             decimals (int, optional): decimal places for float numbers. Defaults to 3.
         """
-        df = self.to_dataframe(confs=confs)
+        df = self.to_df(confs=confs)
         df.to_csv(path, index=False, float_format=f'%.{decimals}f')
 
 
-    @staticmethod
-    def _mask_nnp_ready(mol: Mol, model: str) -> bool:
-        """A mask function to return True if molecule is NNP ready. 
-
-        Args:
-            mol (Mol): rdworks.Mol object.
-            model (str): name of NNP model.
-
-        Returns:
-            bool: True if molecule is NNP ready.
-        """
-        return mol.nnp_ready(model)
-    
-
-    def nnp_ready(self, model: str, **kwargs) -> Self:
-        """Returns a copy of subset of library that is ready to given neural network potential.
-
-        Examples:
-            >>> libr = rdworks.MolLibr(drug_smiles, drug_names)
-            >>> ani2x_compatible_subset = libr.nnp_ready('ANI-2x', progress=False)
-
-        Args:
-            model (str): name of model.
-
-        Returns:
-            Self: subset of library.
-        """
-        obj = self.copy().compute(**kwargs)
-        lmodel = [model,] * self.count()
-        with ProcessPoolExecutor(max_workers=obj.max_workers) as executor:
-            if obj.progress:
-                mask = list(tqdm(
-                    executor.map(self._mask_nnp_ready, obj.libr, lmodel, chunksize=obj.chunksize),
-                    desc="NNP ready",
-                    total=obj.count()))
-            else:
-                mask = list(
-                    executor.map(self._mask_nnp_ready, obj.libr, lmodel, chunksize=obj.chunksize))
-            obj.libr = list(itertools.compress(obj.libr, mask))
-        
-        return obj
-    
-
-    def to_nnbatches(self, batchsize: int = 1000) -> list:
+    def to_batches(self, batchsize_atoms: int = 1000) -> list:
         """Split workload flexibily into a numer of batches.
 
-        - Each batch has up to `batchsize` number of atoms.
+        - Each batch has up to `batchsize_atoms` number of atoms.
         - Conformers originated from a same molecule can be splitted into multiple batches.
         - Or one batch can contain conformers originated from multiple molecules.
 
@@ -999,7 +1023,7 @@ class MolLibr:
         charges: (N,)
 
         Args:
-            batchsize: max. number of atoms in a batch.
+            batchsize_atoms: max. number of atoms in a batch.
 
         Returns:
             list: list of batches.
@@ -1013,7 +1037,7 @@ class MolLibr:
         for mol in self.libr:
             for conf in mol.confs:
                 n_atoms = conf.props['atoms']
-                if (batch_n_atoms + n_atoms) > batchsize:
+                if (batch_n_atoms + n_atoms) > batchsize_atoms:
                     pre_batches.append((batch_mols, batch_confs, batch_n_atoms))
                     # start over a new batch
                     batch_mols =  [mol]
