@@ -7,7 +7,7 @@ import collections
 import operator
 import itertools
 
-from typing import Any, Optional, List, Tuple
+from typing import Any
 
 from rdkit import Chem
 from rdkit.Chem import rdMolDescriptors
@@ -17,7 +17,7 @@ from rdkit.Chem import BRICS, AllChem
 from .std import desalt_smiles
 
 
-def remove_exocyclic(rdmol:Chem.Mol) -> Chem.Mol:
+def remove_exocyclic(rdmol: Chem.Mol) -> Chem.Mol:
     """Removes exocyclic chains or all terminal side chains.
 
     It is equivalent to the `MurckoScaffold.GetScaffoldForMol(mol)`.
@@ -52,7 +52,7 @@ def remove_exocyclic(rdmol:Chem.Mol) -> Chem.Mol:
     return molframe
 
 
-def get_attached_linkers(mol:Chem.Mol) -> Any:
+def get_attached_linkers(mol: Chem.Mol) -> Any:
     """Get linkers (connected non-ring atoms) between rings.
 
     Args:
@@ -77,7 +77,7 @@ def get_attached_linkers(mol:Chem.Mol) -> Any:
     return attached_linkers
 
 
-def breakup(parents:Any, maxChildren:Optional[int]=None, verbose:bool=False) -> List:
+def breakup(parents: Any, maxChildren: int | None = None, verbose: bool = False) -> list:
     """Breaks up parents recursively and return a list of scaffolds.
     
     Examples:
@@ -367,14 +367,14 @@ def breakup(parents:Any, maxChildren:Optional[int]=None, verbose:bool=False) ->
         return parents
     
 
-def scaffold_tree(rdmol:Chem.Mol) -> List[Chem.Mol]:
+def scaffold_tree(rdmol: Chem.Mol) -> list[Chem.Mol]:
     """Returns scaffold tree.
 
     Args:
         rdmol (Chem.Mol): input molecule.
 
     Returns:
-        List[Chem.Mol]: scaffold tree.
+        list[Chem.Mol]: scaffold tree.
     """
     lmol = [rdmol]
     tree = breakup(rdmol, maxChildren=1)
@@ -383,14 +383,14 @@ def scaffold_tree(rdmol:Chem.Mol) -> List[Chem.Mol]:
     return lmol
 
 
-def scaffold_network(rdmol:Chem.Mol) -> List[Chem.Mol]:
+def scaffold_network(rdmol: Chem.Mol) -> list[Chem.Mol]:
     """Returns scaffold network.
 
     Args:
         rdmol (Chem.Mol): input molecule.
 
     Returns:
-        List[Chem.Mol]: scaffold network.
+        list[Chem.Mol]: scaffold network.
     """
     lmol = [rdmol]
     network = breakup(rdmol, maxChildren=None)
@@ -399,18 +399,18 @@ def scaffold_network(rdmol:Chem.Mol) -> List[Chem.Mol]:
     return lmol
 
 
-def BRICS_fragmented(rdmol:Chem.Mol,
-                     min_atoms:Optional[int]=None,
-                     max_atoms:Optional[int]=None) -> List[Chem.Mol]:
+def BRICS_fragmented(rdmol: Chem.Mol,
+                     min_atoms:int | None = None,
+                     max_atoms:int | None = None) -> list[Chem.Mol]:
     """Perform BRICKS decomposition and returns fragmented molecules.
 
     Args:
         rdmol (Chem.Mol): input molecule.
-        min_atoms (Optional[int], optional): min number of atoms for a fragment. Defaults to None.
-        max_atoms (Optional[int], optional): max number of atoms for a fragment. Defaults to None.
+        min_atoms (int, optional): min number of atoms for a fragment. Defaults to None.
+        max_atoms (int, optional): max number of atoms for a fragment. Defaults to None.
 
     Returns:
-        List[Chem.Mol]: a list of fragmented molecules.
+        list[Chem.Mol]: a list of fragmented molecules.
     """
     dummy = Chem.MolFromSmiles('*')
     hydro = Chem.MolFromSmiles('[H]')
@@ -432,23 +432,24 @@ def BRICS_fragmented(rdmol:Chem.Mol,
     return lfrag_rdmol
 
 
-def depth_first_search(rdatom:Chem.Atom, origin_atom:Chem.Atom, 
-                       end_idx:int, group:List[int], BRICS_bonds:List[Tuple[int,int]]) -> List[List[int]]:
+def depth_first_search(rdatom: Chem.Atom, origin_atom: Chem.Atom, 
+                       end_idx:int, group:list[int], BRICS_bonds:list[tuple[int,int]]) -> list[list[int]]:
     """Does recursive depth-first search.
 
     Args:
         rdatom (Chem.Atom): input atom.
         origin_atom (Chem.Atom): origin atom.
         end_idx (int): end index.
-        group (List[int]): group to be appended by the function.
-        BRICS_bonds (List[Tuple[int,int]]): list of bonds(tuple of two indexes)
+        group (list[int]): group to be appended by the function.
+        BRICS_bonds (list[tuple[int,int]]): list of bonds(tuple of two indexes)
 
     Returns:
-        List[List[int]]: search output.
+        list[list[int]] or None: search output.
     """
     bonded_atoms = rdatom.GetNeighbors()
     if (len(bonded_atoms) == 1) and (bonded_atoms[0] == origin_atom):
         return
+    
     for atom in bonded_atoms:
         idx = atom.GetIdx()
         if (idx == end_idx) or (idx in group) or (sorted([rdatom.GetIdx(), idx]) in BRICS_bonds):
@@ -457,14 +458,14 @@ def depth_first_search(rdatom:Chem.Atom, origin_atom:Chem.Atom,
         depth_first_search(atom, rdatom, end_idx, group, BRICS_bonds)
 
 
-def BRICS_fragment_indices(rdmol:Chem.Mol) -> List[List[int]]:
+def BRICS_fragment_indices(rdmol: Chem.Mol) -> list[list[int]]:
     """Returns BRICS fragment/scaffold atom indices.
 
     Args:
         rdmol (Chem.Mol): input molecule.
 
     Returns:
-        List[List[int]]: fragment/scaffold atom indices.
+        list[list[int]]: fragment/scaffold atom indices.
     """
     BRICS_bonds = [sorted(x[0]) for x in list(BRICS.FindBRICSBonds(rdmol))]
     if BRICS_bonds:
@@ -485,7 +486,7 @@ def BRICS_fragment_indices(rdmol:Chem.Mol) -> List[List[int]]:
     return sorted(indices, key=lambda x: len(x), reverse=True)
 
 
-def rigid_fragment_indices(rdmol:Chem.Mol) -> List[List[int]]:
+def rigid_fragment_indices(rdmol: Chem.Mol) -> list[list[int]]:
     """Breaks a molecule at each rotatable bond and returns atom indices of fragments.
     Args:
         rdmol (Chem.Mol) : input molecule
@@ -495,6 +496,7 @@ def rigid_fragment_indices(rdmol:Chem.Mol) -> List[List[int]]:
     """
     rotatable_bond_pattern = Chem.MolFromSmarts('[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]')
     rotatable_bonds = [sorted(x) for x in list(rdmol.GetSubstructMatches(rotatable_bond_pattern))]
+    connecting_atom_indices = [b[0] for b in rotatable_bonds] + [b[1] for b in rotatable_bonds]
     if rotatable_bonds:
         indices = []
         for bond in rotatable_bonds:
@@ -508,13 +510,16 @@ def rigid_fragment_indices(rdmol:Chem.Mol) -> List[List[int]]:
                     depth_first_search(atom, origin_atom, end_idx, group, rotatable_bonds)
                 if sorted(group) not in indices:
                     indices.append(sorted(group))
-    else: # all indices
+    else: 
+        # all indices
         indices = [ [a.GetIdx() for a in rdmol.GetAtoms()] ]
-    # ignore H
+    
+    # remove H atom indices
     indices_noH = []
     for ii in indices:
         indices_noH.append([i for i in ii if rdmol.GetAtomWithIdx(i).GetAtomicNum() != 1 ])
     
-    return sorted(indices_noH, key=lambda x: len(x), reverse=True)
-
-
+    # fragment with more connections and more number of atoms will be prioritized
+    return sorted(indices_noH, 
+                  key=lambda x: (sum([connecting_atom_indices.count(i) for i in x]), len(x)), 
+                  reverse=True)
\ No newline at end of file
