@@ -4,7 +4,7 @@ from rdkit import Chem
 from rdworks.xtb.wrapper import GFN2xTB
 
 
-def get_torsion_atoms(rdmol:Chem.Mol, strict:bool=True) -> list[tuple]:
+def get_torsion_angle_atom_indices(rdmol:Chem.Mol, strict:bool=True) -> list[tuple]:
     """Determine dihedral angle atoms (a-b-c-d) and rotating group for each rotatable bond (b-c).
 
     Args:
