@@ -2,7 +2,10 @@ import numpy as np
 import math
 import operator
 import gzip
+import zlib
 import base64
+import binascii
+import json
 
 from pathlib import Path
 from types import SimpleNamespace
@@ -14,31 +17,101 @@ from tqdm import tqdm
 from rdworks.autograph.centroid import centroid_medoid
 
 
-def compress_string(text: str) -> str:
+def serialize(data: Any) -> str:
+    """
+    Serialize, compress, and encode data to a base64 string.
+
+    Notes:
+        The JSON specification only supports string keys in objects.
+        For example, after JSON-serialization/deserialization, keys of integer type are changed to string.
+        {1: 'a', 2: 'b', 3: 'c'} --> {'1': 'a', '2': 'b', '3': 'c'}
+        Unfortunately, this is a fundamental limitation of JSON itself. 
+        Integer keys are not valid JSON.
+
+    Args:
+        data: Any JSON-serializable Python object
+        
+    Returns:
+        Base64-encoded string
+    """
+    # 1. Serialize to JSON string
+    json_str = json.dumps(data, separators=(',', ':'))  # Compact format
+    
+    # 2. Encode to bytes
+    json_bytes = json_str.encode('utf-8')
+    
+    # 3. Compress
+    compressed = zlib.compress(json_bytes)
+    
+    # 4. Base64 encode (no need to decode to str, keep as bytes if storing in binary)
+    # Base64 output only contains: A-Z, a-z, 0-9, +, /, =
+    encoded = base64.b64encode(compressed)
+    
+    # 5. Convert to string for text storage/transmission
+    return encoded.decode('utf-8')
+
+
+def deserialize(encoded_str: str) -> Any:
+    """
+    Decode, decompress, and deserialize a base64 string back to Python object.
+    
+    Args:
+        encoded_str: Base64-encoded compressed JSON string
+        
+    Returns:
+        Deserialized Python object
+    """
+    try:
+        # 1. Convert string to bytes
+        encoded_bytes = encoded_str.encode('utf-8')
+        
+        # 2. Base64 decode
+        # Base64 output only contains: A-Z, a-z, 0-9, +, /, =
+        compressed = base64.b64decode(encoded_bytes)
+        
+        # 3. Decompress
+        json_bytes = zlib.decompress(compressed)
+        
+        # 4. Decode bytes to string
+        json_str = json_bytes.decode('utf-8')
+        
+        # 5. Parse JSON
+        return json.loads(json_str)
+        
+    except (zlib.error, binascii.Error, json.JSONDecodeError, UnicodeDecodeError) as e:
+        raise ValueError(f"Failed to deserialize data: {e}")
+
+
+def compress_string(data: str) -> str:
     """compress string to base64-encoded string.
 
     Args:
-        text (str): original string.
+        data (str): original string.
 
     Returns:
-        str: base64-encoded gzip compressed string.
+        str: base64-encoded compressed string.
     """
-    compressed_bytes = gzip.compress(text.encode('utf-8'))
-    return base64.b64encode(compressed_bytes).decode('utf-8')
+    compressed_bytes = zlib.compress(data.encode('utf-8'))
+    encoded_str = base64.b64encode(compressed_bytes).decode('utf-8')
+    return encoded_str
 
 
-def decompress_string(compressed_text: str) -> str:
+def decompress_string(encoded_str: str) -> str:
     """decompress base64-encoded string to original string.
 
     Args:
-        compressed_text (str): base64-encoded gzip compressed string.
+        encoded_str (str): base64-encoded compressed string.
 
     Returns:
         str: original string.
     """
-    compressed_bytes = base64.b64decode(compressed_text)
-    decompressed_text = gzip.decompress(compressed_bytes).decode('utf-8')
-    return decompressed_text
+    # automatically add missing padding
+    missing_padding = len(encoded_str) % 4
+    if missing_padding:
+        encoded_str += '=' * (4 - missing_padding)
+    decoded_bytes = base64.b64decode(encoded_str)
+    decompressed = zlib.decompress(decoded_bytes)
+    return decompressed.decode('utf-8')
 
 
 def compute(fn:Callable, largs: list, **kwargs) -> list:
@@ -62,65 +135,6 @@ def compute(fn:Callable, largs: list, **kwargs) -> list:
 
 
 
-def precheck_path(path:str | Path) -> Path:
-    """Prechecks filename or path and returns a string for the pathlib.PosixPath.
-
-    Args:
-        path (Union[str, PosixPath]): filename or path.
-        origin (str): origin of data.
-
-    Raises:
-        FileNotFoundError: if the path is not found.
-
-    Returns:
-        str: a string for the path.
-    """
-    if isinstance(path, Path):
-        pass
-    elif isinstance(path, str):
-        path = Path(path)
-    if path.exists() and path.is_file():
-        return path
-    else:
-        raise FileNotFoundError(f"File path {path.as_posix()} does not exist.")
-    
-
-
-def guess_mol_id(lprops:list[dict]) -> tuple[str, int, int]:
-    """Guesses molecular ID from SDF properties.
-
-    Molecular ID is guessed by the coverage(=count of unique values divided by total count).
-    A property suitable for ID should have coverage of 1.0.
-
-    Args:
-        lprops (List[dict]): a list of properties.
-
-    Returns:
-        Tuple[str, int, int]: (property, count of unique values, total count)
-    """
-    f = {} # unique occurrence
-    for props in lprops:
-        for k in props:
-            v = props[k]
-            # float is not suitable for molecular id
-            if isinstance(v, float): 
-                continue 
-            if not (k in f): 
-                f[k] = set()
-            # str(int) is acceptable for molecular id
-            if isinstance(v, int): 
-                f[k].add(str(v))
-            elif isinstance(v, str):
-                f[k].add(v)
-    r = [(k, len(f[k]), -max([len(x) for x in f[k]])) for k in f]
-    r = sorted(r, key=operator.itemgetter(1,2))
-    try:
-        (property_key, count, total) = (r[-1][0], r[-1][1], len(lprops))
-    except IndexError:
-        (property_key, count, total) = (None, 0, 0)
-    return (property_key, count, total)
-
-
 def dict_to_simplenamespace(data):
     if isinstance(data, dict):
         return SimpleNamespace(**{k: dict_to_simplenamespace(v) for k, v in data.items()})
@@ -245,10 +259,10 @@ def _QT_diameter(rmsd_matrix:np.ndarray, A:list) -> float:
     Returns:
         float: maximum pairwise distance.
     """
-    return np.max(rmsd_matrix[A][:,A])
+    return np.max(rmsd_matrix[A][:,A]).item()
 
 
-def _QT_clustering(rmsd_matrix:np.ndarray, G:set, threshold:float, clusters:list) -> list:
+def _QT_clustering(rmsd_matrix: np.ndarray, G: set, threshold: float, clusters: list) -> None:
     """A subroutine for `QT()` to perform QTC algorithm.
 
     Args:
@@ -264,6 +278,7 @@ def _QT_clustering(rmsd_matrix:np.ndarray, G:set, threshold:float, clusters:list
     if len(G) <= 1:
         clusters.append(G)
         return
+    
     C = [] # cluster candidates
     for i in G:
         flag = True
@@ -289,7 +304,7 @@ def _QT_clustering(rmsd_matrix:np.ndarray, G:set, threshold:float, clusters:list
     _QT_clustering(rmsd_matrix, G-largest_C, threshold, clusters)
 
 
-def QT(rmsd_matrix:np.ndarray, threshold:float) -> tuple:
+def QT(rmsd_matrix: np.ndarray, threshold: float) -> tuple:
     """Perform QT clustering.
 
     Args:
@@ -308,6 +323,7 @@ def QT(rmsd_matrix:np.ndarray, threshold:float) -> tuple:
         for conf_idx in indices:
             cluster_assignment[conf_idx] = cluster_idx
     centroid_indices = centroid_medoid(cluster_assignment, rmsd_matrix)
+    
     return cluster_assignment, centroid_indices
 
 
