@@ -32,7 +32,7 @@ def test_torsion_fragment():
     
     mol = Mol(molecule="CC(C)C1=C(C(=C(N1CC[C@H](C[C@H](CC(=O)O)O)O)C2=CC=C(C=C2)F)C3=CC=CC=C3)C(=O)NC4=CC=CC=C4",
               name="atorvastatin").make_confs(n=1)
-    ta = mol.torsion_atoms()
+    ta = mol.get_torsion_angle_atoms()
     assert len(ta) == 12
     # {0: (0, 1, 3, 7),  1: (3, 4, 32, 33),  2: (4, 5, 26, 27),  3: (7, 6, 19, 20),
     # 4: (3, 7, 8, 9),  5: (7, 8, 9, 10),   6: (8, 9, 10, 18),   7: (18, 10, 11, 12),
@@ -43,7 +43,7 @@ def test_torsion_fragment():
     assert wbo_filtered == True
 
     mol2 = Mol(molecule='CC(=O)Nc1ccc(O)cc1', name='acetaminophen.3').make_confs(n=1)
-    ta2 = mol2.torsion_atoms()
+    ta2 = mol2.get_torsion_angle_atoms()
     # {0: (5, 4, 3, 1)}
     assert len(ta2) == 1
 
@@ -59,7 +59,7 @@ def test_torsion_fragment_from_conf():
     mol = Mol(molecule="CC(C)C1=C(C(=C(N1CC[C@H](C[C@H](CC(=O)O)O)O)C2=CC=C(C=C2)F)C3=CC=CC=C3)C(=O)NC4=CC=CC=C4",
               name="atorvastatin").make_confs(n=1)
     ref_conf = mol.confs[0]
-    ta = ref_conf.torsion_atoms()
+    ta = ref_conf.get_torsion_angle_atoms()
     assert len(ta) == 12
     # {0: (0, 1, 3, 7),  1: (3, 4, 32, 33),  2: (4, 5, 26, 27),  3: (7, 6, 19, 20),
     # 4: (3, 7, 8, 9),  5: (7, 8, 9, 10),   6: (8, 9, 10, 18),   7: (18, 10, 11, 12),
@@ -69,11 +69,11 @@ def test_torsion_fragment_from_conf():
     assert frag_created == True
     assert wbo_filtered == True
 
-    ref_conf = ref_conf.torsion_energies(calculator='MMFF94', torsion_key=6, interval=15)
+    ref_conf = ref_conf.calculate_torsion_energies(calculator='MMFF94', torsion_angle_idx=6, interval=15)
 
     mol2 = Mol(molecule='CC(=O)Nc1ccc(O)cc1', name='acetaminophen.3').make_confs(n=1)
     ref_conf2 = mol2.confs[0]
-    ta2 = ref_conf2.torsion_atoms()
+    ta2 = ref_conf2.get_torsion_angle_atoms()
     # {0: (5, 4, 3, 1)}
     assert len(ta2) == 1
     frag, frag_ijkl, frag_created, wbo_filtered = create_torsion_fragment(ref_conf2.rdmol, ta2[0])
@@ -83,19 +83,18 @@ def test_torsion_fragment_from_conf():
     assert frag_created == False
     assert wbo_filtered == False
 
-    ref_conf2 = ref_conf2.torsion_energies(calculator='MMFF94', interval=15)
-    ref_conf3 = ref_conf2.torsion_energies_one(calculator='MMFF94', indices=frag_ijkl)
+    ref_conf2 = ref_conf2.calculate_torsion_energies(calculator='MMFF94', interval=15)
+    ref_conf3 = ref_conf2.calculate_torsion_energies_one(calculator='MMFF94', indices=frag_ijkl)
 
 
 def test_torsion_energies():
     libr = MolLibr(torsion_dataset_smiles, torsion_dataset_names)
-    with open(workdir / 'test_torsion_energies.html', 'w', encoding='utf-8') as f:
-        for mol in libr[:1]:
-            mol = mol.make_confs().drop_confs(similar=True, similar_rmsd=0.3).sort_confs().rename()
-            mol = mol.optimize_confs(calculator='MMFF94').torsion_energies(calculator='MMFF94', 
-                                                                           interval=15)
-            f.write(mol.to_html())
-            print(mol.dumps('torsion', decimals=2))
+    for mol in libr[:1]:
+        mol = mol.make_confs().drop_confs(similar=True, similar_rmsd=0.3).sort_confs().rename()
+        mol = mol.optimize_confs(calculator='MMFF94')
+        mol = mol.calculate_torsion_energies(calculator='MMFF94', interval=15)
+        plot_data = mol.to_plot_data_torsion_angle_vs_energy()
+        print(mol.dumps('torsion', decimals=2))
 
 
 def test_torsion_energies_batch():
@@ -116,10 +115,12 @@ def test_torsion_energies_batch():
             return Optimized(mols=self.rdmols)
     
     libr = MolLibr(torsion_dataset_smiles, torsion_dataset_names)
-    with open(workdir / 'test_torsion_energies_batch.html', 'w', encoding='utf-8') as f:
-        for mol in libr[:1]:
-            mol = mol.make_confs().drop_confs(similar=True, similar_rmsd=0.3).sort_confs().rename()
-            mol = mol.optimize_confs(calculator='MMFF94')
-            mol = mol.torsion_energies(Dummy_BatchOptimizer, interval=15, batchsize_atoms=16384)
-            f.write(mol.to_html())
-            print(mol.dumps('torsion', decimals=2))
+    for mol in libr[:1]:
+        mol = mol.make_confs().drop_confs(similar=True, similar_rmsd=0.3).sort_confs().rename()
+        mol = mol.optimize_confs(calculator='MMFF94')
+        mol = mol.calculate_torsion_energies(
+            Dummy_BatchOptimizer, 
+            interval=15, 
+            batchsize_atoms=16384)
+        plot_data = mol.to_plot_data_torsion_angle_vs_energy()
+        print(mol.dumps('torsion', decimals=2))
